import numpy as np

# vypocet urciteho integralu lichobeznikovym pravidlem

def quad_trapezoid(f,a,b,N):

    h=(b-a)/N
    S=(f(a)+f(b))/2

    for j in range(1,N):
        S=S+f(a+h*j)

    return h*S


# vypocet urciteho integralu Simpsonovym pravidlem, N musi byt sude

def quad_Simpson(f,a,b,N):

    h=(b-a)/N
    S=f(a)+f(b)

    for j in range(1,N,2):
        S=S+4*f(a+h*j)
    for j in range(2,N-1,2):
        S=S+2*f(a+h*j)

    return (h/3)*S


# vypocet urciteho integralu Rombergovou integraci
# - varianta s koncovymi body, N=2^lev

def quad_Romberg(f,a,b,levmin,levmax,kmax,acc):

    T=np.zeros((levmax+1,kmax+1))

    sum1=(f(a)+f(b))/2
    sum2=0.0
    for lev in range(levmax+1):

        # vypocet lichobeznikovym pravidlem
        N=2**lev
        h=(b-a)/N
        for j in range(1,N,2): # 1,N-1,2
            sum2=sum2+f(a+h*j)
        T[lev,0]=h*(sum1+sum2)
    
        # Richardsonova extrapolace
        for k in range(1,min(lev,kmax)+1):
            T[lev,k]=(4**k*T[lev,k-1]-T[lev-1,k-1])/(4**k-1)
        val=T[lev,min(lev,kmax)]
    
        if (lev>=levmin):
            if np.abs(val-prev)<acc: break
        prev=val

    return val,N


#-----------------------------------------------------------

def fun(x): return x/(1+np.sin(x))

acc=1e-6
lmax=10
kmax=4

I1,N=quad_Romberg(fun,0.0,np.pi/2,3,lmax,kmax,acc)
I2=quad_Simpson(fun,0.0,np.pi/2,N)
I3=quad_trapezoid(fun,0.0,np.pi/2,N)
I=np.log(2.0)

print( "%s%6d" % ("N =",N) )
print( "%s%20.15f,%12.2e" % ("Romberg   :",I1,(I1-I)/I ))
print( "%s%20.15f,%12.2e" % ("Simpson   :",I2,(I2-I)/I ))
print( "%s%20.15f,%12.2e" % ("trapezoid :",I3,(I3-I)/I ))
print( "%s%20.15f"        % ("ln(2)     :",I           ))
